/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2010-2013
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** 
 *   @file  csl_cppi.h
 *
 *   @brief  
 *      This is the CPPI device specific include file. This file defines various queue 
 *      types
 */

/** @defgroup CSL_CPPI_API CPPI
 *
 */
#ifndef CSL_CPPI_H_
#define CSL_CPPI_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <ti/csl/csl.h>

/**
@addtogroup CPPI_LLD_SYMBOL
@{
*/

/** CPPI maximum number of CPDMAs */
#define CPPI_MAX_CPDMA     13

/**
@}
*/

/**
@addtogroup CPPI_LLD_ENUM
@{
*/

/** 
 * @brief CPPI CPDMA types
 */
typedef enum
{
    /** SRIO */
    Cppi_CpDma_SRIO_CPDMA = 0,
    /** AIF */
    Cppi_CpDma_AIF_CPDMA,
    /** FFTC A */
    Cppi_CpDma_FFTC_A_CPDMA,
    /** FFTC B */
    Cppi_CpDma_FFTC_B_CPDMA,
    /** FFTC C */
    Cppi_CpDma_FFTC_C_CPDMA,
    /** FFTC D */
    Cppi_CpDma_FFTC_D_CPDMA,
    /** FFTC E */
    Cppi_CpDma_FFTC_E_CPDMA,
    /** FFTC F */
    Cppi_CpDma_FFTC_F_CPDMA,
    /** PASS */
    Cppi_CpDma_PASS_CPDMA,
    /** QMSS in first QM */
    Cppi_CpDma_QMSS_CPDMA,
    /** QMSS in second QM */
    Cppi_CpDma_QMSS_QM2_CPDMA,
    /** BCP */
    Cppi_CpDma_BCP_CPDMA,
    /** XGE */
    Cppi_CpDma_XGE_CPDMA
}Cppi_CpDma;

/**
@}
*/



#ifdef __cplusplus
}
#endif

#endif /* CSL_CPPI_H_ */
