/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2011-2013
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** 
 *   @file  csl_qm_queue.h
 *
 *   @brief  
 *      This is the Queue Manager device specific include file. This file 
 *      defines various queue types
 * 
 */

/** @defgroup CSL_QMSS_API QMSS
 *
 */
#ifndef __CSL_QMSS_QUEUE_H__
#define __CSL_QMSS_QUEUE_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <ti/csl/csl.h>

#if defined(DEVICE_K2K)

#include <ti/csl/device/k2k/src/csl_qm_queue.h>

#elif defined(DEVICE_K2H)

#include <ti/csl/device/k2h/src/csl_qm_queue.h>

#endif /* DEVICE_XXXXX */

/** 
 * @brief Queue Type. Specifies different queue classifications
 */
typedef enum
{
    /** Low priority queue */
    Qmss_QueueType_LOW_PRIORITY_QUEUE = 0,
    /** AIF queue */
    Qmss_QueueType_AIF_QUEUE,
    /** PASS 0 queue */
    Qmss_QueueType_PASS_QUEUE,
    /** INTC pending queue (from SET1) */
    Qmss_QueueType_INTC_QUEUE,
    /** INTC SET2 queue */
    Qmss_QueueType_INTC_SET2_QUEUE,
    /** INTC SET3 queue */
    Qmss_QueueType_INTC_SET3_QUEUE,
    /** INTC SET4 queue */
    Qmss_QueueType_INTC_SET4_QUEUE,
    /** SRIO queue */
    Qmss_QueueType_SRIO_QUEUE,
    /** FFTC queue A */
    Qmss_QueueType_FFTC_A_QUEUE,
    /** FFTC queue B */
    Qmss_QueueType_FFTC_B_QUEUE,
    /** FFTC queue C */
    Qmss_QueueType_FFTC_C_QUEUE,
    /** FFTC queue D */
    Qmss_QueueType_FFTC_D_QUEUE,
    /** FFTC queue E */
    Qmss_QueueType_FFTC_E_QUEUE,
    /** FFTC queue F */
    Qmss_QueueType_FFTC_F_QUEUE,
    /** BCP queue */
    Qmss_QueueType_BCP_QUEUE,
    /** High priority queue */
    Qmss_QueueType_HIGH_PRIORITY_QUEUE,
    /** starvation counter queue */
    Qmss_QueueType_STARVATION_COUNTER_QUEUE,
    /** Infrastructure queue first QM */
    Qmss_QueueType_INFRASTRUCTURE_QUEUE,
    /** Infrastructure queue second QM */
    Qmss_QueueType_QM2_INFRASTRUCTURE_QUEUE,
    /** Traffic shaping queue */
    Qmss_QueueType_TRAFFIC_SHAPING_QUEUE,
    /** GIC400 queue */
    Qmss_QueueType_GIC400_QUEUE,
    /** EDMA 4 queue */
    Qmss_QueueType_EDMA_4_QUEUE,
    /** Broadcast to Hyperlink 0 and 1 queue */
    Qmss_QueueType_HLINK_BROADCAST_QUEUE,
    /** Hyperlink 0 queue */
    Qmss_QueueType_HLINK_0_QUEUE,
    /** Hyperlink 1 queue */
    Qmss_QueueType_HLINK_1_QUEUE,
    /** XGE (10 gigabit ethernet) queue */
    Qmss_QueueType_XGE_QUEUE,
    /** DXB queue */
    Qmss_QueueType_DXB_QUEUE,
    /** General purpose queue -- MUST be last */
    Qmss_QueueType_GENERAL_PURPOSE_QUEUE
}Qmss_QueueType;

#ifdef __cplusplus
}
#endif

#endif /* __CSL_QMSS_QUEUE_H__ */
