/********************************************************************
* Copyright (C) 2003-2008 Texas Instruments Incorporated.
* All Rights Reserved
*********************************************************************
* file: cslr_bcp_cor.h
*
* Brief: This file contains the Register Description for bcp_cor
*
*********************************************************************/
#ifndef _CSLR_BCP_COR_MMR_H_
#define _CSLR_BCP_COR_MMR_H_

#include <ti/csl/cslr.h>

#include <ti/csl/tistdtypes.h>


/* Minimum unit = 1 byte */

/**************************************************************************\
* Register Overlay Structure for cor Interrupt Registers
\**************************************************************************/
typedef struct  {
    volatile Uint32 COR_INTR_IRS; /* BCP Raw Status Register */
    volatile Uint32 COR_INTR_IRS_SET; /* BCP Set Raw Status Register */
    volatile Uint32 COR_INTR_IRS_CLR; /* BCP Clear Raw Status Register */
    volatile Uint32 COR_INTR_EN; /* BCP Enable cor_intr0 Register */
    volatile Uint32 COR_INTR_EN_SET; /* BCP Set Enable cor_intr0 Register */
    volatile Uint32 COR_INTR_EN_CLR; /* BCP Clear Enable cor_intr0 Register */
    volatile Uint32 COR_INTR_EN_STS; /* BCP Enabled cor_intr0 Register */
} CSL_Bcp_corCor_interrupt_registersRegs; /* COR interrupt register group */

/**************************************************************************\
* Register Overlay Structure for COR Data Logger
\**************************************************************************/
typedef struct  {
    volatile Uint32 COR_DLG_GLOBAL_HDR; /* Data logger control Register */
    volatile Uint32 COR_DLG_TM_SW_TIMESTAMP; /* Data logger control Register */
    volatile Uint32 COR_DLG_HW_TIMESTAMP_ENG_ERR; /* Data logger control Register */
    volatile Uint8 RSVD0[4];
} CSL_Bcp_corCor_data_loggerRegs; /* Data Logger */

/**************************************************************************\
* Register Overlay Structure
\**************************************************************************/
typedef struct  {
    volatile Uint32 M0; /* M0 */
    volatile Uint32 M1; /* M1 */
    volatile Uint32 M2; /* M2 */
    volatile Uint32 M3; /* M3 */
    volatile Uint32 M4; /* M4 */
    volatile Uint32 M5; /* M5 */
    volatile Uint32 M6; /* M6 */
    volatile Uint32 M7; /* M7 */
    volatile Uint32 M8; /* M8 */
    volatile Uint32 M9; /* M9 */
    volatile Uint32 M10; /* M10 */
    volatile Uint32 M11; /* M11 */
    volatile Uint32 M12; /* M12 */
    volatile Uint32 CONTROL_REGISTER; /* control register */
    volatile Uint8 RSVD1[72];
    CSL_Bcp_corCor_interrupt_registersRegs COR_INTERRUPT_REGISTERS[4];
    volatile Uint32 COR_DATA_LOGGER_CTL; /* Data logger control Register */
    volatile Uint32 COR_DATA_LOGGER_STATUS; /* Datalogger Status Register */
    volatile Uint32 COR_GLOBAL_HDR; /* Submodule Global Header Register */
    volatile Uint8 RSVD2[41732];
    CSL_Bcp_corCor_data_loggerRegs COR_DATA_LOGGER[256];
} CSL_Bcp_corRegs;

/**************************************************************************\
* Field Definition Macros
\**************************************************************************/

/* cor_intr_irs */

#define CSL_BCP_COR_COR_INTR_IRS_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_IRS_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_IRS_RESETVAL (0x00000000u)

/* cor_intr_irs_set */

#define CSL_BCP_COR_COR_INTR_IRS_SET_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_SET_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_IRS_SET_RESETVAL (0x00000000u)

/* cor_intr_irs_clr */

#define CSL_BCP_COR_COR_INTR_IRS_CLR_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_IRS_CLR_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_IRS_CLR_RESETVAL (0x00000000u)

/* cor_intr_en */

#define CSL_BCP_COR_COR_INTR_EN_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_EN_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_EN_RESETVAL (0x00000000u)

/* cor_intr_en_set */

#define CSL_BCP_COR_COR_INTR_EN_SET_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_SET_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_EN_SET_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_SET_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_SET_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_SET_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_EN_SET_RESETVAL (0x00000000u)

/* cor_intr_en_clr */

#define CSL_BCP_COR_COR_INTR_EN_CLR_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_CLR_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_EN_CLR_RESETVAL (0x00000000u)

/* cor_intr_en_sts */

#define CSL_BCP_COR_COR_INTR_EN_STS_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_STS_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_INTR_EN_STS_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_STS_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_COR_COR_INTR_EN_STS_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E0_COR_A_GT_13_MASK (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E0_COR_A_GT_13_SHIFT (0x00000002u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E0_COR_A_GT_13_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E1_COR_A_ZERO_MASK (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E1_COR_A_ZERO_SHIFT (0x00000003u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E1_COR_A_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E2_COR_NR_GT_8_MASK (0x00000010u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E2_COR_NR_GT_8_SHIFT (0x00000004u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E2_COR_NR_GT_8_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E3_COR_NR_ZERO_MASK (0x00000020u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E3_COR_NR_ZERO_SHIFT (0x00000005u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E3_COR_NR_ZERO_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E4_COR_DATA_SHORT_MASK (0x00000040u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E4_COR_DATA_SHORT_SHIFT (0x00000006u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E4_COR_DATA_SHORT_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E5_DSPRD_DATA_SIZE_ERR_MASK (0x00000080u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E5_DSPRD_DATA_SIZE_ERR_SHIFT (0x00000007u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E5_DSPRD_DATA_SIZE_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E6_UNUSED_MASK (0x00000100u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E6_UNUSED_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_INTR_EN_STS_COR_E6_UNUSED_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_INTR_EN_STS_RESETVAL (0x00000000u)

/* COR_dlg_global_hdr */

#define CSL_BCP_COR_COR_DLG_GLOBAL_HDR_GLOBAL_HDR_MASK (0xFFFFFFFFu)
#define CSL_BCP_COR_COR_DLG_GLOBAL_HDR_GLOBAL_HDR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_DLG_GLOBAL_HDR_GLOBAL_HDR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DLG_GLOBAL_HDR_RESETVAL (0x00000000u)

/* COR_dlg_tm_sw_timestamp */

#define CSL_BCP_COR_COR_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_MASK (0xFFFFFFFFu)
#define CSL_BCP_COR_COR_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DLG_TM_SW_TIMESTAMP_RESETVAL (0x00000000u)

/* COR_dlg_hw_timestamp_eng_err */

#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_MASK (0xFF000000u)
#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_SHIFT (0x00000018u)
#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_MASK (0x00FFFFFFu)
#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DLG_HW_TIMESTAMP_ENG_ERR_RESETVAL (0x00000000u)

/* M0 */


#define CSL_BCP_COR_M0_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M0_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M0_MOD_EN_RESETVAL   (0x000FFFFFu)

#define CSL_BCP_COR_M0_RESETVAL          (0x000FFFFFu)

/* M1 */


#define CSL_BCP_COR_M1_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M1_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M1_MOD_EN_RESETVAL   (0x0005A933u)

#define CSL_BCP_COR_M1_RESETVAL          (0x0005A933u)

/* M2 */


#define CSL_BCP_COR_M2_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M2_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M2_MOD_EN_RESETVAL   (0x00010E5Au)

#define CSL_BCP_COR_M2_RESETVAL          (0x00010E5Au)

/* M3 */


#define CSL_BCP_COR_M3_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M3_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M3_MOD_EN_RESETVAL   (0x0006339Cu)

#define CSL_BCP_COR_M3_RESETVAL          (0x0006339Cu)

/* M4 */


#define CSL_BCP_COR_M4_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M4_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M4_MOD_EN_RESETVAL   (0x0007C3E0u)

#define CSL_BCP_COR_M4_RESETVAL          (0x0007C3E0u)

/* M5 */


#define CSL_BCP_COR_M5_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M5_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M5_MOD_EN_RESETVAL   (0x000FFC00u)

#define CSL_BCP_COR_M5_RESETVAL          (0x000FFC00u)

/* M6 */


#define CSL_BCP_COR_M6_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M6_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M6_MOD_EN_RESETVAL   (0x000D8E64u)

#define CSL_BCP_COR_M6_RESETVAL          (0x000D8E64u)

/* M7 */


#define CSL_BCP_COR_M7_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M7_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M7_MOD_EN_RESETVAL   (0x0004F5B0u)

#define CSL_BCP_COR_M7_RESETVAL          (0x0004F5B0u)

/* M8 */


#define CSL_BCP_COR_M8_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M8_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M8_MOD_EN_RESETVAL   (0x000218ECu)

#define CSL_BCP_COR_M8_RESETVAL          (0x000218ECu)

/* M9 */


#define CSL_BCP_COR_M9_MOD_EN_MASK       (0x000FFFFFu)
#define CSL_BCP_COR_M9_MOD_EN_SHIFT      (0x00000000u)
#define CSL_BCP_COR_M9_MOD_EN_RESETVAL   (0x0001B746u)

#define CSL_BCP_COR_M9_RESETVAL          (0x0001B746u)

/* M10 */


#define CSL_BCP_COR_M10_MOD_EN_MASK      (0x000FFFFFu)
#define CSL_BCP_COR_M10_MOD_EN_SHIFT     (0x00000000u)
#define CSL_BCP_COR_M10_MOD_EN_RESETVAL  (0x0000FFFFu)

#define CSL_BCP_COR_M10_RESETVAL         (0x0000FFFFu)

/* M11 */


#define CSL_BCP_COR_M11_MOD_EN_MASK      (0x000FFFFFu)
#define CSL_BCP_COR_M11_MOD_EN_SHIFT     (0x00000000u)
#define CSL_BCP_COR_M11_MOD_EN_RESETVAL  (0x00033FFFu)

#define CSL_BCP_COR_M11_RESETVAL         (0x00033FFFu)

/* M12 */


#define CSL_BCP_COR_M12_MOD_EN_MASK      (0x000FFFFFu)
#define CSL_BCP_COR_M12_MOD_EN_SHIFT     (0x00000000u)
#define CSL_BCP_COR_M12_MOD_EN_RESETVAL  (0x0003FFFCu)

#define CSL_BCP_COR_M12_RESETVAL         (0x0003FFFCu)

/* control_register */


#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K0_MASK (0x000000C0u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K0_SHIFT (0x00000006u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K0_RESETVAL (0x00000000u)

#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K1_MASK (0x00000030u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K1_SHIFT (0x00000004u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K10_C2K1_RESETVAL (0x00000000u)

#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K0_MASK (0x0000000Cu)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K0_SHIFT (0x00000002u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K0_RESETVAL (0x00000000u)

#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K1_MASK (0x00000003u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K1_SHIFT (0x00000000u)
#define CSL_BCP_COR_CONTROL_REGISTER_QPSK_MAPPING_C2K11_C2K1_RESETVAL (0x00000000u)

#define CSL_BCP_COR_CONTROL_REGISTER_RESETVAL (0x00000000u)

/* COR_data_logger_ctl */

#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_CTL_MASK (0x00000007u)
#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_CTL_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_CTL_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_ERR_MASK_MASK (0x01FF0000u)
#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_ERR_MASK_SHIFT (0x00000010u)
#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_DLG_ERR_MASK_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_DATA_LOGGER_CTL_RESETVAL (0x00000000u)

/* COR_data_logger_status */

#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_PTR_MASK (0x000000FFu)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_PTR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_PTR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_WRAP_MASK (0x0000FF00u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_WRAP_SHIFT (0x00000008u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_WR_WRAP_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_MASK (0x00010000u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_SHIFT (0x00000010u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_RUNNING_MASK (0x00020000u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_RUNNING_SHIFT (0x00000011u)
#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_DLG_RUNNING_RESETVAL (0x00000000u)


#define CSL_BCP_COR_COR_DATA_LOGGER_STATUS_RESETVAL (0x00000000u)

/* COR_global_hdr */

#define CSL_BCP_COR_COR_GLOBAL_HDR_GLOBAL_HDR_MASK (0xFFFFFFFFu)
#define CSL_BCP_COR_COR_GLOBAL_HDR_GLOBAL_HDR_SHIFT (0x00000000u)
#define CSL_BCP_COR_COR_GLOBAL_HDR_GLOBAL_HDR_RESETVAL (0x00000000u)

#define CSL_BCP_COR_COR_GLOBAL_HDR_RESETVAL (0x00000000u)

#endif
