/*  ===========================================================================
 *  Copyright (c) Texas Instruments Incorporated 2002-2011
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** 
 *   @file  cslr_bcr_cfg.h
 *
 *   @brief  
 *      This file contains the Register Desciptions for BCR_CFG
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2002, 2003, 2004, 2005, 2006, Texas Instruments, Inc.
 *  @n   Use of this software is controlled by the terms and conditions found 
 *  @n   in the license agreement under which this software has been supplied.
 *  ===========================================================================
 *  \par 
 */

#ifndef _CSLR_BCR_CFG_H_
#define _CSLR_BCR_CFG_H_

#include <ti/csl/cslr.h>
#include <ti/csl/tistdtypes.h>


/* Minimum unit = 4 bytes */

/**************************************************************************\
* Register Overlay Structure
\**************************************************************************/
typedef struct  {
    volatile Uint32 BCR_PID;
    volatile Uint32 BCR_RAC_POP;
    volatile Uint32 RSVD0[6];
    volatile Uint32 BCR_MST_CFG[8];
} CSL_Rac2_cfgRegs;

/**************************************************************************\
* Field Definition Macros
\**************************************************************************/

/* BCR_PID */


#define CSL_BCR_CFG_BCR_PID_RESETVAL     (0x48300000u)

/* BCR_RAC_POP */


#define CSL_BCR_CFG_BCR_RAC_POP_POP_7_MASK  (0x00000080u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_7_SHIFT (0x00000007u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_7_RESETVAL (0x00000000u)
/*----POP_7 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_7_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_7_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_6_MASK  (0x00000040u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_6_SHIFT (0x00000006u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_6_RESETVAL (0x00000000u)
/*----POP_6 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_6_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_6_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_5_MASK  (0x00000020u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_5_SHIFT (0x00000005u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_5_RESETVAL (0x00000000u)
/*----POP_5 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_5_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_5_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_4_MASK  (0x00000010u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_4_SHIFT (0x00000004u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_4_RESETVAL (0x00000000u)
/*----POP_4 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_4_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_4_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_3_MASK  (0x00000008u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_3_SHIFT (0x00000003u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_3_RESETVAL (0x00000000u)
/*----POP_3 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_3_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_3_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_2_MASK  (0x00000004u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_2_SHIFT (0x00000002u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_2_RESETVAL (0x00000000u)
/*----POP_2 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_2_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_2_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_1_MASK  (0x00000002u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_1_SHIFT (0x00000001u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_1_RESETVAL (0x00000000u)
/*----POP_1 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_1_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_1_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_POP_0_MASK  (0x00000001u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_0_SHIFT (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_0_RESETVAL (0x00000000u)
/*----POP_0 Tokens----*/
#define CSL_BCR_CFG_BCR_RAC_POP_POP_0_NOT_POPULATED (0x00000000u)
#define CSL_BCR_CFG_BCR_RAC_POP_POP_0_POPULATED (0x00000001u)

#define CSL_BCR_CFG_BCR_RAC_POP_RESETVAL     (0x00000000u)

/* BCR_MST_CFG */


#define CSL_BCR_CFG_BCR_MST_CFG_MEM_REG_ID_MASK (0x00000007u)
#define CSL_BCR_CFG_BCR_MST_CFG_MEM_REG_ID_SHIFT (0x00000000u)
#define CSL_BCR_CFG_BCR_MST_CFG_MEM_REG_ID_RESETVAL (0x00000000u)

#define CSL_BCR_CFG_BCR_MST_CFG_RESETVAL (0x00000000u)


#endif
