/** 
 *   @file  fftc_testcfg_48.h
 *
 *   @brief  
 *      Header file containing FFTC Test vector details.
 *
 *      The format of the file contents are as follows:
 *          *   Line 1:                         Number of FFT data blocks in this
 *                                              packet configuration.
 *          *   Line 2 onwards (32 entries):    FFT configuration for the data, details
 *                                              below:
 *                  (01)    DFT sample size - each sample is again 4 bytes wide.
 *                          (Fftc_QLocalCfg.controlRegConfig.dftSize)
 *                  (02)    Not used
 *                  (03)    Not used
 *                  (04)    Not used
 *                  (05)    Dynamic Scaling Enable Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bDynamicScaleEnable)
 *                  (06)    Last Radix Stage Output Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingValLast)
 *                  (07)    Radix Stage 0 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[0])
 *                  (08)    Radix Stage 1 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[1])
 *                  (09)    Radix Stage 2 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[2])
 *                  (10)    Radix Stage 3 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[3])
 *                  (11)    Radix Stage 4 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[4])
 *                  (12)    Radix Stage 5 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[5])
 *                  (13)    Radix Stage 6 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[6])
 *                  (14)    Not used
 *                  (15)    Not used
 *                  (16)    Not used
 *                  (17)    Inverse FFT enable boolean flag
 *                          (Fftc_QLocalCfg.controlRegConfig.bInverseFFTEnable)
 *                  (18)    Emulate DSP 16x16 boolean flag - Not used currently
 *                          (Fftc_QLocalCfg.controlRegConfig.bEmulateDSP16x16)
 *                  (19)    Output Scaling Value
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.outputScaleVal)
 *                  (20)    Input Shifting Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bInputFFTShift)
 *                  (21)    Output Shifting Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bOutputFFTShift)
 *                  (22)    Zero Pad Mode Enable boolean flag
 *                          (Fftc_QLocalCfg.controlRegConfig.bZeroPadEnable)
 *                  (23)    Zero Pad Mode (used only if zero padding enabled) 
 *                          (Fftc_QLocalCfg.controlRegConfig.zeroPadMode)
 *                  (24)    Zero Pad Factor (used only if zero padding enabled) 
 *                          (Fftc_QLocalCfg.controlRegConfig.zeroPadFactor)
 *                  (25)    LTE Frequency Shifting Enable boolean flag
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.bFreqShiftEnable)
 *                  (26)    LTE Frequency Shift value (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftVal)
 *                  (27)    LTE Frequency Shift Table Index (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftIndex)
 *                  (28)    LTE Frequency Shift Multiplication factor (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftMultFactor)
 *                  (29)    LTE Frequency Shift Phase (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftInitPhase)
 *                  (30)    LTE Frequency Shift Direction (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftDirection)
 *                  (31)    Cyclic Prefix Addition Enable Boolean flag 
 *                          (Fftc_QLocalCfg.cyclicPrefixRegConfig.bCyclicPrefixAddEnable)
 *                  (32)    Cyclic Prefix Addition Value (Used only if cyclic prefix addition enabled) 
 *                          (Fftc_QLocalCfg.cyclicPrefixRegConfig.cyclicPrefixAddNum)
 *          *   Block Number
 *          *   FFT input data (sample size * 2 entries)
 *          *   Expected Block Exponent Value
 *          *   Expected Clipping Detect Value
 *          *   Expected Number of clock cycles to be taken for operation          
 *          *   Expected FFT output data - reference data to validate result from FFT engine.
 *              (sample size * 2 entries)
 * 
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2009, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _FFTC_TESTCFG_48_H_
#define _FFTC_TESTCFG_48_H_

Int16 inputConfig_48[] = {
4,
48,
16,
19,
15,
1,
0,
0,
0,
0,
0,
0,
0,
0,
1,
1,
1,
0,
0,
128,
0,
0,
0,
0,
1,
0,
0,
0,
0,
0,
0,
0,
0,
1,
2271,
4223,
-21,
-10587,
-2707,
-2595,
4333,
-2668,
-364,
5433,
227,
-544,
2851,
4033,
-3092,
4192,
2339,
6716,
-606,
859,
2300,
-3669,
515,
2745,
1381,
4482,
5585,
2521,
-4202,
-28,
-9592,
3591,
5924,
5421,
1575,
26,
301,
-1044,
2154,
1502,
3367,
3256,
2529,
427,
-4545,
3061,
3683,
2342,
1156,
-4040,
1443,
5250,
9406,
-587,
2078,
-617,
-5918,
-5404,
5297,
-6900,
6975,
-3916,
2666,
-254,
-7496,
1070,
3749,
-4959,
407,
7651,
2936,
440,
3593,
7660,
-4541,
6201,
343,
-6357,
2748,
2107,
-4163,
-4945,
1160,
-736,
-826,
-1155,
946,
-2191,
-696,
-2381,
296,
-3695,
1713,
5716,
-3752,
2124,
2,
0,
30,
12475,
-656,
-2253,
3992,
7815,
2223,
10998,
-386,
-1617,
3336,
-1430,
-2102,
-458,
-11101,
-3951,
-1277,
2207,
2389,
-6138,
-13338,
3010,
-5234,
-237,
60,
-4372,
10415,
4268,
-5817,
1285,
10197,
-647,
2647,
-6175,
11426,
-9232,
-3126,
1271,
-761,
-2803,
9209,
2685,
7172,
16707,
-5118,
2287,
-3334,
-3782,
-5476,
2401,
-322,
16432,
1009,
6900,
-930,
-13132,
4006,
-1886,
-2051,
-2145,
1160,
-8854,
-4606,
15846,
-4636,
10973,
-8000,
-6447,
5340,
496,
8862,
-8595,
5434,
7360,
3683,
-11004,
-10334,
-195,
1395,
-8283,
4447,
10471,
-2193,
15517,
5555,
3588,
9926,
-3438,
690,
335,
-14415,
-3680,
-9631,
1106,
-6310,
4728,
-2699,
2,
-4982,
-649,
-2436,
2144,
6054,
1417,
7005,
8892,
-14,
53,
680,
1534,
2044,
-388,
5345,
3609,
-3960,
4932,
-3167,
6683,
-4800,
6057,
-5391,
2236,
5718,
6000,
-4602,
-2411,
3100,
2833,
2237,
-3354,
3097,
-6981,
23,
-3503,
-690,
9306,
-5801,
1661,
1505,
-5870,
4026,
-6331,
1141,
6387,
-2311,
1128,
3990,
264,
-4077,
-12469,
-5428,
-12815,
3311,
-3809,
-4327,
-3344,
5682,
-8516,
-1113,
3702,
-7668,
-4432,
-3569,
7156,
376,
-2269,
694,
-4974,
-673,
7716,
-44,
1597,
-2136,
-3193,
4012,
442,
-4163,
-1691,
6589,
4350,
2305,
-7462,
5147,
-2320,
5358,
8734,
3133,
-8417,
3178,
3208,
-3249,
6574,
4804,
-2179,
2,
0,
30,
9802,
-13754,
-9429,
-10229,
-10902,
-8603,
-6665,
-10477,
5368,
8837,
-3152,
-5744,
7796,
-2550,
5387,
11629,
12674,
8908,
-4578,
-15125,
-4088,
-7328,
-14620,
7286,
-7891,
2148,
3294,
7858,
-7870,
-11985,
6102,
872,
-5443,
-14639,
6673,
7594,
-1493,
-11931,
5887,
-21549,
3777,
703,
-345,
1255,
4541,
-686,
9592,
13311,
-5504,
-13181,
-4665,
-1601,
1791,
10604,
19016,
-3249,
-160,
3529,
3376,
-2752,
-2818,
-4452,
-7360,
8758,
-1553,
5330,
-540,
16385,
-35,
-13550,
4760,
853,
11570,
14181,
-12782,
5756,
-653,
-5376,
-12882,
7652,
8503,
11271,
1789,
-17361,
2587,
-25,
5928,
1055,
3167,
-5708,
115,
1398,
7045,
1277,
2981,
3411,
3,
-4357,
-690,
5774,
-2904,
15978,
1520,
2525,
3935,
-273,
-2281,
-4322,
2224,
-1421,
-4221,
-11318,
3283,
1276,
-4879,
-4243,
-18,
1312,
-1236,
1638,
-2210,
3473,
-3931,
-5186,
4887,
940,
8218,
3349,
-1347,
-3125,
-3364,
-5983,
5091,
-1338,
5795,
-5807,
-880,
4147,
-7104,
-8030,
-3359,
5118,
-9204,
-1851,
-564,
1856,
5630,
4797,
-2978,
1514,
-8984,
-3424,
-1653,
-765,
-2602,
6581,
2970,
-5956,
-4272,
-1821,
8222,
6077,
3465,
8902,
2681,
7382,
-861,
-3705,
-3592,
3867,
2683,
-1593,
6376,
-6861,
-10604,
-1859,
803,
-4987,
-4394,
-3553,
12021,
1054,
-3575,
3748,
7257,
-2846,
4751,
1808,
-4031,
-6332,
306,
2989,
369,
2,
0,
30,
-6241,
3897,
-9025,
5801,
334,
-3768,
3279,
-4199,
21,
-10431,
10554,
1623,
2071,
11112,
3290,
-2999,
-5544,
-15055,
2256,
-9067,
-3967,
-8788,
12046,
99,
379,
16515,
13932,
3941,
-9723,
-6000,
-2496,
-10342,
-4521,
-12398,
-16408,
-2442,
25522,
-4754,
-8981,
8803,
843,
-3814,
-22487,
9669,
-10102,
-11036,
19663,
-3347,
4215,
-2135,
-14986,
-5279,
-646,
9029,
14802,
-7619,
5297,
-1809,
-10090,
-782,
2662,
-7938,
12110,
1093,
327,
-13754,
-1737,
12490,
-6066,
3540,
3364,
4802,
-779,
-884,
7085,
-4415,
-3846,
-4767,
-3338,
-7246,
8639,
-842,
5769,
-7300,
-12096,
-3546,
4084,
-24,
-8326,
4204,
2674,
8187,
13083,
13116,
12137,
-188,
4,
12726,
-2561,
5623,
7187,
4458,
-8966,
7854,
4553,
1632,
8175,
5142,
-3601,
3210,
-2780,
1453,
-2149,
2310,
2681,
3699,
-6996,
5156,
7077,
4632,
1014,
-5692,
-3596,
6536,
-2377,
-1507,
-1325,
-5868,
4813,
1964,
-2809,
7288,
2580,
-2971,
-3379,
-6504,
679,
-9778,
4429,
-2474,
1797,
-2092,
7539,
-3488,
1699,
-2658,
9855,
5101,
801,
1913,
-2796,
4993,
-2692,
-436,
-2953,
262,
2960,
2395,
-2262,
-2249,
2932,
-3625,
8273,
4279,
2766,
-3406,
23,
3453,
-8510,
1457,
-9200,
1314,
4073,
2472,
4392,
-5675,
3039,
-1032,
-6997,
5613,
-431,
-1884,
190,
4031,
4378,
4475,
-2063,
-829,
11584,
16,
4113,
-5852,
-5310,
2,
0,
30,
11748,
4469,
1376,
6114,
8899,
2025,
10795,
10427,
-4801,
19615,
-2850,
11975,
-7373,
520,
18799,
10997,
11596,
18980,
-4856,
-4021,
7512,
-797,
-5305,
-3675,
4906,
-2372,
10582,
17203,
4182,
-450,
-3822,
-204,
2010,
16543,
4951,
334,
135,
4270,
-14293,
-11736,
-9734,
1551,
-5209,
-1671,
-7261,
3737,
10216,
6673,
7573,
-4924,
-6309,
-2557,
3574,
4383,
-13209,
-7597,
-933,
3692,
-933,
9233,
-2510,
716,
3176,
-1654,
1843,
-6658,
12913,
-10342,
11181,
-5407,
7183,
8735,
-509,
-16556,
-7967,
-19137,
5161,
-6240,
-2321,
12809,
-3081,
-9864,
-706,
-6586,
-4867,
508,
710,
3505,
-1777,
3662,
-708,
5489,
-17726,
5800,
3035,
14300,
5,
5912,
-248,
4472,
-4115,
-6136,
-7296,
4115,
-4705,
-6154,
-5031,
1032,
-5411,
-3298,
5271,
-4988,
-5938,
1261,
5438,
2875,
78,
1520,
-3319,
-2238,
-3645,
3663,
-2443,
1277,
1776,
2420,
5155,
419,
-1205,
4029,
-7512,
-2504,
2327,
2223,
-681,
-6172,
7044,
-5114,
-3220,
-137,
-6300,
3055,
4944,
2736,
-9074,
2121,
553,
3367,
-41,
1018,
-763,
-6698,
-4891,
836,
-7703,
5123,
-6830,
-3103,
3869,
3839,
-3187,
-208,
-7194,
2492,
3647,
3289,
-5670,
-5071,
4551,
1470,
-4018,
1322,
750,
3957,
-10654,
859,
2228,
3171,
-3507,
1228,
-5588,
-569,
-7207,
-2991,
-5500,
-2676,
1719,
733,
-11898,
-4912,
4476,
1396,
-3804,
2,
0,
30,
-8461,
-2186,
6446,
6644,
1272,
12034,
229,
-9753,
8205,
-11270,
-11265,
14845,
-761,
36,
3878,
17402,
7254,
-6658,
-3116,
-626,
961,
6496,
6653,
4762,
10595,
3287,
13539,
1067,
137,
-8764,
13143,
9138,
3923,
7556,
3218,
-22940,
2008,
-13035,
1888,
1490,
2143,
3420,
-6443,
1310,
-1775,
8230,
3780,
-8989,
-14167,
1849,
9633,
11001,
91,
5448,
866,
1259,
-5347,
-16851,
-9096,
-11653,
-5358,
1070,
9123,
-2047,
-1551,
356,
5615,
6556,
-10210,
3131,
-245,
639,
19164,
-4600,
-9913,
1278,
8370,
7733,
-4921,
-1302,
-1590,
9879,
8489,
-5837,
4755,
4146,
5219,
-208,
-552,
1629,
3474,
-5884,
6151,
4726,
3546,
-8411
};

#endif  /* _FFTC_TESTCFG_48_H_ */
