#*******************************************************************************
#* FILE PURPOSE: Lower level makefile for Creating Component Libraries for ARMv7
#*******************************************************************************
#* FILE NAME: ./lib/libsa_aearmv7.mk
#*
#* DESCRIPTION: Defines Source Files, Compilers flags and build rules
#*
#*******************************************************************************
#

#
# Macro definitions referenced below
#
empty =
space =$(empty) $(empty)

# Output for prebuilt generated libraries
ARMV7SALIBDIR ?= $(SA_INSTALL_PATH)/ti/drv/sa/lib/armv7
ARMV7SAOBJDIR ?= ./obj
ARMV7SAOBJDIR_SO := $(ARMV7SAOBJDIR)/sa/lib_so
ARMV7SAOBJDIR := $(ARMV7SAOBJDIR)/sa/lib
ARMV7SABINDIR ?= ./bin
DEBUG_FLAG    ?= -O2

ifdef CROSS_TOOL_INSTALL_PATH
# Support backwards compatibility with KeyStone1 approach
CC = $(CROSS_TOOL_INSTALL_PATH)/$(CROSS_TOOL_PRFX)gcc
AC = $(CROSS_TOOL_INSTALL_PATH)/$(CROSS_TOOL_PRFX)as
AR = $(CROSS_TOOL_INSTALL_PATH)/$(CROSS_TOOL_PRFX)ar
LD = $(CROSS_TOOL_INSTALL_PATH)/$(CROSS_TOOL_PRFX)gcc
endif

INCS = -I. -I$(strip $(subst ;, -I,$(subst $(space),\$(space),$(INCDIR))))

CFLAGS+=-mno-unaligned-access
INTERNALDEFS = -D__ARMv7 -D_LITTLE_ENDIAN=1 -DMAKEFILE_BUILD
CFLAGS += $(INTERNALDEFS)

OBJEXT = o 
INTERNALLINKDEFS =
SRCDIR = ./src

VPATH=$(SRCDIR) 

#List the COMMONSRC Files
COMMONSRCC = \
    salld.c\
    salldcom.c\
    salldtxrx.c\
    salldctbl.c\
    salldsc.c\
    salldmci.c\
    proto/srtp/salldsrtp.c\
    proto/srtp/salldsrtpinit.c\
    proto/srtp/salldsrtp_ctr.c\
    proto/srtp/salldsrtp_f8.c\
    proto/srtp/salldsrtpsc.c\
    proto/srtp/salldsrtcp.c\
    proto/srtp/salldsrtcpinit.c\
    proto/srtp/salldsrtcp_ctr.c\
    proto/srtp/salldsrtcp_f8.c\
    proto/ipsec/salldipsecinit.c\
    proto/ipsec/salldipsec.c\
    proto/ipsec/salldah.c\
    proto/ipsec/salldesp.c\
    proto/airciph/salldacinit.c\
    proto/airciph/salldac.c\
    proto/datamode/sallddminit.c\
    proto/datamode/sallddm.c\
    proto/datamode/sallddmcmdl.c\
    cipher/salldaes.c\
	cipher/salldaes_ctr.c\
    cipher/salldaes_f8.c\
    cipher/salldgcm.c\
    auth/salldcmac.c\
    auth/salldhmac_md5.c\
    auth/salldhmac_sha1.c\
    auth/salldhmac_sha2.c\
    auth/salldmd5.c\
	auth/salldsha1.c\
    auth/salldsha2.c\
    auth/salldxcbc.c

# FLAGS for the COMMONSRC Files
COMMONSRCCFLAGS = $(DEBUG_FLAG) -I$(SRCDIR) -I.
CFLAGS +=$(COMMONSRCCFLAGS)


# Make Rule for the COMMONSRC Files
COMMONSRCCOBJS = $(patsubst %.c, $(ARMV7SAOBJDIR)/%.$(OBJEXT), $(COMMONSRCC))
COMMONSRCCOBJS_SO = $(patsubst %.c, $(ARMV7SAOBJDIR_SO)/%.$(OBJEXT), $(COMMONSRCC))

$(COMMONSRCCOBJS): $(ARMV7SAOBJDIR)/%.$(OBJEXT): %.c $(ARMV7SAOBJDIR)/.created
	-@echo compiling $< ...
	@if [ ! -d $(@D) ]; then mkdir -p $(@D) ; fi;
	@$(CC) -c $(CFLAGS) $(INCS)  $< -o $@
	
$(COMMONSRCCOBJS_SO): $(ARMV7SAOBJDIR_SO)/%.$(OBJEXT): %.c $(ARMV7SAOBJDIR_SO)/.created
	-@echo compiling $< ...
	@if [ ! -d $(@D) ]; then mkdir -p $(@D) ; fi;	
	@$(CC) -c $(CFLAGS) -fPIC $(INCS)  $< -o $@

$(ARMV7SALIBDIR)/libsa.a: $(COMMONSRCCOBJS) $(ARMV7SALIBDIR)/.created
	@echo archiving static library $? into $@ ...
	@$(AR) -r $@ $?

libsa.so: $(COMMONSRCCOBJS_SO)
	@echo archiving $? into $(ARMV7SALIBDIR)/$@.1 ...
	@$(CC) $(DEBUG_FLAG) -ggdb2 -Wl,-soname=$@.1 -shared -fPIC ${LDFLAGS} -o $@.1.0.0 $^
	@ln -s $@.1.0.0 $@.1
	@ln -s $@.1     $@	
	@mv -f $@.1.0.0 $(ARMV7SALIBDIR)/$@.1.0.0
	@mv -f $@.1 $(ARMV7SALIBDIR)/$@.1
	@mv -f $@   $(ARMV7SALIBDIR)/$@	

$(ARMV7SAOBJDIR)/.created:
	@mkdir -p $(ARMV7SAOBJDIR)
	@touch $(ARMV7SAOBJDIR)/.created

$(ARMV7SAOBJDIR_SO)/.created:
	@mkdir -p $(ARMV7SAOBJDIR_SO)
	@touch $(ARMV7SAOBJDIR_SO)/.created

$(ARMV7SALIBDIR)/.created:
	@mkdir -p $(ARMV7SALIBDIR)
	@touch $(ARMV7SALIBDIR)/.created

clean:
	@echo cleaning $(ARMV7SAOBJDIR)
	@$(RMDIR) $(ARMV7SAOBJDIR)
	@$(RMDIR) $(ARMV7SAOBJDIR_SO)
